<?php
namespace Teknisa\CADEST\Modules\Produto\Controllers;

use Teknisa\CADEST\Modules\Produto\Services\Produto as ProdutoService;
use Teknisa\CADEST\Modules\Produto\Services\Fiscal as FiscalService;
use Teknisa\CADEST\Modules\Produto\Services\ProdutoUtilities;
use Teknisa\Libs\Service\CRUD as CrudService;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;
use React\Promise\FulfilledPromise;
use React\Promise\Deferred;
use \Zeedhi\Framework\DTO;
use Teknisa\Libs\Util\Environment;

class Produto {

    protected $produtoService;
    protected $fiscalService;
    protected $utilities;
    protected $crudService;
    protected $environment;
    protected $baseUri;

    public function __construct(ProdutoService $produtoService, FiscalService $fiscalService,
                                 ProdutoUtilities $utilities, CrudService $crudService, Environment $environment,
                                 $dataSourceManager, $baseUri) {
        $this->produtoService   = $produtoService;
        $this->fiscalService    = $fiscalService; 
        $this->utilities        = $utilities;
        $this->crudService      = $crudService;
        $this->environment      = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->baseUri          = $baseUri;
    }

    public function save(Row $request, Response $response) {
        $row = $request->getRow();
        
        $row["CDARVPROD"] = str_replace(".", "", $row["CDARVPROD"]);
        
        $levelMask = strlen(str_replace(".", "", $this->produtoService->getProductMask()));
        try {
            $this->utilities->beginTransaction();
            if (isset($row['__is_new']) && $row['__is_new'] === true) {
                str_replace('.', '', $row['CDARVPROD']);
                $this->produtoService->existArvprod($row['CDARVPROD']);
                $this->rulesToNewProduct($row);
                $row = $this->crudService->getRowsToSave($request, 'produto')[0];
                $this->utilities->flush();
            } else {
                $this->rulesToEditProduct($row);
            }
            $row['CDPRODNIVANTE'] = $row['CDARVPRODNIVANTE'] ? $row['CDARVPRODNIVANTE'] : $row['CDPRODNIVANTE'];
            $row['CDPRODNIVANTE']? $this->produtoService->arvprodContainsProdnivante($row['CDARVPROD'], $row['CDARVPRODNIVANTE']) : null;
            $this->produtoService->correctLevelByArvprod($row['CDARVPROD'], $row['CDNVPRODUTO']);
            $this->utilities->saveInProduto($row);
            
            $lengthMask  = strlen(str_replace(".", "", $this->produtoService->getProductMask()));
            if($row["__is_new"] && strlen($row["CDARVPROD"]) == $lengthMask) {
                $this->produtoService->callInsereAgrupa($row["CDARVPROD"]);
            }
            $this->utilities->flush();
            if (isset($row['__is_new']) && $row['__is_new'] === true) {
                $this->fiscalService->saveFiscal($row);
            }
            $this->utilities->commit();
            
            /*if($row["__is_new"] == true) {
                try {
                    $lengthMask  = strlen(str_replace(".", "", $this->produtoService->getProductMask()));
                    if(strlen($row["CDARVPROD"]) == $lengthMask) {
                        //$this->callSaveProdutoFilial($row); Função desativada até futura definição pois foi criada uma trigger que realiza o mesmo processo para atender o delphi ao mesmo
                    }
                }
                catch(\Exception $e) {
                    $response->addMessage(new Response\Message("Não foi possivel replicar o cadastro do produto para as filiais."));
                    $response->addMessage(new Response\Message($e->getMessage()));
                }
            }*/
            
            
            $row['__is_new'] = false;
            $dataSet = new DataSet('produto', array($row));
            $response->setDataSets(array($dataSet));
        } catch (\Exception $e) {
            $this->utilities->rollBack();
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
            
    }
    
    public function delete(Row $request, Response $response) {
        try{
            
            $row = $request->getRow();
            $this->produtoService->deleteProduct($row);
            
            
        }catch(\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage()));
        }
       
    }
    
    /**
     *  Chama uma requisição assincrona que replica o cadastro do produto inserido para todas as filiais.
     */
    
    public function callSaveProdutoFilial($row) { //Função desativada até futura definição pois foi criada uma trigger que realiza o mesmo processo para atender o delphi ao mesmo

        $requestUri = $_SERVER["REQUEST_URI"];
        
        if(strpos($requestUri, "modules") == false) {
            $urlSaveProdFili = $_SERVER["HTTP_HOST"].'/backend_cadEst/index.php/saveProdutoFilial';
        }
        else {
            if(strpos($_SERVER["DOCUMENT_ROOT"], "workfolder") != false) {
                $urlSaveProdFili = $_SERVER["HTTP_HOST"].'/workfolder'.$_SERVER["REQUEST_URI"];
            }
            else {
                $urlSaveProdFili = $_SERVER["HTTP_HOST"].$_SERVER["REQUEST_URI"];
            }
        }
        
        $urlSaveProdFili = 'http://'.str_replace("saveProduct", "saveProdutoFilial", $urlSaveProdFili);

        $parameters = array("requestType" => "Row", "row" => 
                            array("CDPRODUTO"   => str_replace(".", "", $row["CDARVPROD"]),
                                  "CDOPERADOR"  => $this->environment->getCdOperador(),
                                  "NRORG"       => $this->environment->getNrOrgTrab()));
            

        $parameters = base64_encode(json_encode($parameters));
        
        $file = __DIR__.DIRECTORY_SEPARATOR.'AsyncRequest.php';
        
        $tokenLogin = $this->environment->getLoginTokenInSession();
            
        exec( "php $file $urlSaveProdFili $parameters $tokenLogin > /tmp/logReplicacaoProduto.txt &");
    }
    
    private function rulesToEditProduct(&$row) {
        if ($this->produtoService->productType($row) === 'C') {
            $this->produtoService->validateType($row['CDPRODUTO'], 'C');
            $row = $this->produtoService->applyRulesToCompraType($row);
        } else if ($this->produtoService->productType($row) === 'E') {
            $this->produtoService->validateType($row['CDPRODUTO'], 'E');
            $row = $this->produtoService->applyRulesToEstoqueType($row);
        }
    }

    private function rulesToNewProduct(&$row) {
        $row['CDPRODUTO'] = $this->produtoService->getNewCdproduto($row['CDARVPROD']);
        $row['CDNVPRODUTO'] = $this->produtoService->getCdnvprodutoByCdarvprod($row['CDARVPROD']);
        if ($this->produtoService->newProductType($row) === 'C') {
            $row = $this->produtoService->applyRulesToCompraType($row);
        } else if ($this->produtoService->newProductType($row) === 'E') {
            $row = $this->produtoService->applyRulesToEstoqueType($row);
        }
    }
    
    public function asyncRequest($url, $payload = '', $debugMode = false) {

    	$values = array();
    	
    	error_reporting(E_ALL);
        
        /* Get the port for the WWW service. */
        $service_port = getservbyname('www', 'tcp');
        $url     = parse_url($url);
        
        /* Get the IP address for the target host. */
        $address = $url['host'];
        /* Create a TCP/IP socket. */
        $socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
        if ($socket === false) {
            throw new \Exception("Não foi possivel replicar o cadastro do produto para as filiais");
        } 
        
        $result = socket_connect($socket, $address, $service_port);
        
        if ($result === false) {
            throw new \Exception("Não foi possivel replicar o cadastro do produto para as filiais");
        } 
        
        $tokenLogin = $this->environment->getLoginTokenInSession();
    
    	$in = "POST ".$url['path']." HTTP/1.1\r\n";
    	$in.= "Host: ".$url['host']."\r\n";
    	$in.= "Content-Type: application/json\r\n";
    	$in.= "OAuth-Token: $tokenLogin\r\n";
    	$in.= "Content-Length: ".strlen($payload)."\r\n";
    	$in.= "Connection: Close\r\n\r\n";
    	if ($payload) $in.= $payload;
    	$out = '';

        socket_write($socket, $in, strlen($in));
        
        if($debugMode === true) {
            while ($out = socket_read($socket, 2048)) {
                var_dump($out);
            }
        }
    
        socket_close($socket);
    }
    
    public function saveProdutoFilial(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();

        $dataSetProdFili = $this->produtoService->getDataSetProdutoFilial($row["CDPRODUTO"], $row["CDOPERADOR"], $row["NRORG"]);
        
        foreach($dataSetProdFili as $prodFili) {
            $dataSet = new DataSet('prodfili_insert', $prodFili);
            $this->dataSourceManager->persist($dataSet);
        }
        
    }
    

    
}